﻿using MotorPlantContracts.BindingModels;
using MotorPlantContracts.SearchModels;
using MotorPlantContracts.StoragesContracts;
using MotorPlantContracts.ViewModels;
using MotorPlantDataModels.Models;
using MotorPlantFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MotorPlantFileImplement.Implements
{
	public class ShopStorage : IShopStorage
	{
		private readonly DataFileSingleton _source;
		public ShopStorage()
		{
			_source = DataFileSingleton.GetInstance();
		}
		public List<ShopViewModel> GetFullList()
		{
			return _source.Shops.Select(x => x.GetViewModel).ToList();
		}
		public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
		{
			if (string.IsNullOrEmpty(model.Name))
			{
				return new();
			}
			return _source.Shops.Where(x => x.ShopName.Contains(model.Name)).Select(x => x.GetViewModel).ToList();

		}
		public ShopViewModel? GetElement(ShopSearchModel model)
		{
			if (string.IsNullOrEmpty(model.Name) && !model.Id.HasValue)
			{
				return null;
			}
			return _source.Shops.FirstOrDefault(x => (!string.IsNullOrEmpty(model.Name) && x.ShopName == model.Name) || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
		}
		public ShopViewModel? Insert(ShopBindingModel model)
		{
			model.Id = _source.Shops.Count > 0 ? _source.Shops.Max(x => x.Id) + 1 : 1;
			var newShop = Shop.Create(model);
			if (newShop == null)
			{
				return null;
			}
			_source.Shops.Add(newShop);
			_source.SaveShops();
			return newShop.GetViewModel;
		}
		public ShopViewModel? Update(ShopBindingModel model)
		{
			var component = _source.Shops.FirstOrDefault(x => x.Id == model.Id);
			if (component == null)
			{
				return null;
			}
			component.Update(model);
			_source.SaveShops();
			return component.GetViewModel;
		}
		public ShopViewModel? Delete(ShopBindingModel model)
		{
			var element = _source.Shops.FirstOrDefault(x => x.Id == model.Id);
			if (element != null)
			{
				_source.Shops.Remove(element);
				_source.SaveShops();
				return element.GetViewModel;
			}
			return null;
		}

		public bool SellEngines(IEngineModel model, int count)
		{
			var eng = _source.Engines.FirstOrDefault(x => x.Id == model.Id);
            int store = _source.Shops.SelectMany(x => x.ShopEngines).Sum(y => y.Key == model.Id ? y.Value.Item2 : 0);

            if (eng == null || store < count)
			{
				return false;
			}

			for (int i = 0; i < _source.Shops.Count; i++)
			{
				var shop = _source.Shops[i];
				var engines = shop.ShopEngines;
				foreach (var engine in engines.Where(x => x.Value.Item1.Id == eng.Id))
				{
					var selling = Math.Min(engine.Value.Item2, count);
					engines[engine.Value.Item1.Id] = (engine.Value.Item1, engine.Value.Item2 - selling);

					count -= selling;

					if (count <= 0)
					{
						break;
					}
				}
				shop.Update(new ShopBindingModel
				{
					Id = model.Id,
					ShopName = shop.ShopName,
					Adress = shop.Adress,
					MaxCount = shop.MaxCount,
					DateOpen = shop.DateOpen,
					ShopEngines = engines
				});
			}
			_source.SaveShops();
			return true;
		}
	}
}
