﻿using MotorPlantContracts.BindingModels;
using MotorPlantContracts.ViewModels;
using MotorPlantDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace MotorPlantFileImplement.Models
{
	public class Shop : IShopModel
	{
		public int Id { get; private set; }
		public string ShopName { get; private set; }
		public string Adress { get; private set; }
		public DateTime DateOpen { get; private set; }
		public int MaxCount { get; private set; }
		public Dictionary<int, int> Engines { get; private set; } = new();
		private Dictionary<int, (IEngineModel, int)>? _shopEngines = null;

		public Dictionary<int, (IEngineModel, int)> ShopEngines
		{
			get
			{
				if (_shopEngines == null)
				{
					var source = DataFileSingleton.GetInstance();
					_shopEngines = Engines.ToDictionary(x => x.Key, y => ((source.Engines.FirstOrDefault(z => z.Id == y.Key) as IEngineModel)!, y.Value));
				}
				return _shopEngines;
			}
		}
		public static Shop? Create(ShopBindingModel model)
		{
			if (model == null)
			{
				return null;
			}
			return new Shop()
			{
				Id = model.Id,
				ShopName = model.ShopName,
				Adress = model.Adress,
				DateOpen = model.DateOpen,
				MaxCount = model.MaxCount,
				Engines = model.ShopEngines.ToDictionary(x => x.Key, x => x.Value.Item2)
			};
		}
		public static Shop? Create(XElement element)
		{
			if (element == null)
			{
				return null;
			}
			return new Shop()
			{
				Id = Convert.ToInt32(element.Attribute("Id")!.Value),
				ShopName = element.Element("ShopName")!.Value,
				Adress = element.Element("Adress")!.Value,
				MaxCount = Convert.ToInt32(element.Element("MaxCount")!.Value),
				DateOpen = Convert.ToDateTime(element.Element("DateOpen")!.Value),
				Engines = element.Element("ShopEngines")!.Elements("ShopEngine").ToDictionary(x => Convert.ToInt32(x.Element("Key")?.Value), x => Convert.ToInt32(x.Element("Value")?.Value))
			};

		}
		public void Update(ShopBindingModel? model)
		{
			if (model == null)
			{
				return;
			}
			ShopName = model.ShopName;
			Adress = model.Adress;
			DateOpen = model.DateOpen;
			MaxCount = model.MaxCount;
			if (model.ShopEngines.Count > 0)
			{
				Engines = model.ShopEngines.ToDictionary(x => x.Key, x => x.Value.Item2);
				_shopEngines = null;
			}
		}
		public ShopViewModel GetViewModel => new()
		{
			Id = Id,
			ShopName = ShopName,
			Adress = Adress,
			DateOpen = DateOpen,
			MaxCount = MaxCount,
			ShopEngines = ShopEngines
		};

		public XElement GetXElement => new XElement("Shop",
			new XAttribute("Id", Id),
			new XElement("ShopName", ShopName),
			new XElement("Adress", Adress),
			new XElement("DateOpen", DateOpen),
			new XElement("MaxCount", MaxCount),
			new XElement("ShopEngines", Engines.Select(x => new XElement("ShopEngine", new XElement("Key", x.Key), new XElement("Value", x.Value))).ToArray()));
	}
}
