﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using MotorPlantDataModels.Models;
using MotorPlantContracts.BusinessLogicsContracts;
using MotorPlantContracts.ViewModels;

namespace MotorPlantView
{
	public partial class FormSell : Form
	{
		private readonly List<EngineViewModel>? _engineList;
		IShopLogic _shopLogic;
		IEngineLogic _engineLogic;
		public FormSell(IEngineLogic engineLogic, IShopLogic shopLogic)
		{
			InitializeComponent();
			_shopLogic = shopLogic;
			_engineLogic = engineLogic;
			_engineList = engineLogic.ReadList(null);
			if (_engineList != null)
			{
				comboBoxEngine.DisplayMember = "EngineName";
				comboBoxEngine.ValueMember = "Id";
				comboBoxEngine.DataSource = _engineList;
				comboBoxEngine.SelectedItem = null;
			}
		}
		public int EngineId
		{
			get
			{
				return Convert.ToInt32(comboBoxEngine.SelectedValue);
			}
			set
			{
				comboBoxEngine.SelectedValue = value;
			}
		}

		public IEngineModel? EngineModel
		{
			get
			{
				if (_engineList == null)
				{
					return null;
				}
				foreach (var elem in _engineList)
				{
					if (elem.Id == EngineId)
					{
						return elem;
					}
				}
				return null;
			}
		}


		public int Count
		{
			get { return Convert.ToInt32(textBoxCount.Text); }
			set
			{ textBoxCount.Text = value.ToString(); }
		}

		private void buttonSave_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(textBoxCount.Text))
			{
				MessageBox.Show("Заполните поле Количество", "Ошибка",
			   MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if (comboBoxEngine.SelectedValue == null)
			{
				MessageBox.Show("Выберите двигатель", "Ошибка",
			   MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}

			try
			{
				int count = Convert.ToInt32(textBoxCount.Text);

				bool res = _shopLogic.MakeSell(
					_engineLogic.ReadElement(new() { Id = Convert.ToInt32(comboBoxEngine.SelectedValue) }),
					count
				);

				if (!res)
				{
					throw new Exception("Ошибка при продаже.");
				}

				MessageBox.Show("Продажа прошла успешно");
				DialogResult = DialogResult.OK;
				Close();

			}
			catch (Exception err)
			{
				MessageBox.Show("Ошибка продажи");
				return;
			}
		}

		private void buttonCancel_Click(object sender, EventArgs e)
		{
			Close();
		}
	}
}
